﻿using Kivii;
using Kivii.Lims;
using Kivii.Lims.Entities;
using Kivii.Linq;
using Njhg.Third.Semir.Entities;
using Njhg.Third.Semir.Extensions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Njhg.Third.Semir.Jobs
{
    public class PullJob : IJob
    {
        public string Name => "自动拉取森马订单";

        public string Description => "拉取森马订单任务调度JOB";

        public IJobContext TaskContext { get; set; }

        public bool Execution()
        {
            #region 设置缓存1分钟内不能连续请求
            var cache = KiviiContext.GetCacheClient();
            var cacheKey = KiviiContext.GetUrnKey("SemirOrderAccept:LockedOneMinute");
            var cacheValue = cache.Get<string>(cacheKey);
            if (!cacheValue.IsNullOrEmpty())
            {
                throw new Exception(cacheValue);
            }
            #endregion
            var baseUrl = TaskContext.Arguments["BaseUrl"];
            baseUrl.ThrowIfNullOrEmpty("未配置参数:BaseUrl");
            var token = TaskContext.Arguments["Token"];
            token.ThrowIfNullOrEmpty("未配置参数:Token");
            var length = TaskContext.Arguments.ContainsKey("Length") ? TaskContext.Arguments["Length"].ToInt() : 200;
            var ownerKvidstr = TaskContext.Arguments["OwnerKvid"];
            DateTime? maxTranslateTime = null;
            var time = TaskContext.Arguments.ContainsKey("DateTime") ? TaskContext.Arguments["DateTime"] : "";
            var ownerKvid = Guid.Empty;
            if (!ownerKvidstr.IsNullOrEmpty()) Guid.TryParse(ownerKvidstr, out ownerKvid);
            var ownerName = TaskContext.Arguments["OwnerName"];

            if (time != null)
            {
                if (DateTime.TryParse(time, out var selectTime))
                {
                    maxTranslateTime = selectTime;
                }
            }
            var conn = KiviiContext.GetOpenedDbConnection<ThirdReportOrder>();
            if (maxTranslateTime == null) maxTranslateTime = conn.Scalar<ThirdReportOrder, DateTime>(o => Sql.Max(o.OperateTime), p => p.ThirdType == typeof(SemirOrder).FullName);
            //DateTime maxTranslateTime = conn.Scalar<ThirdReportOrder, DateTime>(o => Sql.Max(o.OperateTime));
            var client = new JsonServiceClient(baseUrl);
            try
            {
                var response = client.PullSemirOrder(token, maxTranslateTime, null, length);
                cache.Set(cacheKey, "一分钟内不可多次查询!", TimeSpan.FromMinutes(1));

                (response.code != 200).ThrowIfTrue(response.message);
                if (response.data == null)
                {
                    TaskContext.Message = "未查询到数据!";
                    return true;
                }
                if (response.data.results.IsNullOrEmpty())
                {
                    TaskContext.Message = "未查询到数据!";
                    return true;
                }

                var results = response.data.results.AcceptThirdContractOrder(ownerKvid, ownerName, conn);

                TaskContext.Message = $"成功插入{results.Count}/{response.data.results.Count}条数据!";
                return true;
            }
            catch (Exception ex)
            {
                TaskContext.Message = $"{ex.Message}";
                return false;
            }
            finally
            {
                var thread = KiviiContext.NewThread(() =>
                {
                    //没有关联报告的 自动执行符合条件的关联
                    SemirExtension.LinkReports(null, client, token);
                });
                thread.Start();
            }
        }

        public void Dispose()
        {
        }
    }

    public class UploadJob : IJob
    {
        public string Name => "自动上传森马订单";

        public string Description => "上传森马订单任务调度JOB";

        public IJobContext TaskContext { get; set; }

        public bool Execution()
        {
            var conn = KiviiContext.GetOpenedDbConnection<ThirdReportOrder>();

            try
            {
                //var querySemir = conn.From<ThirdContractOrder>();
                //querySemir.Where(o => o.ThirdType == typeof(SemirOrder).FullName & o.IsUploaded == false & o.IsAccepted == true);
                //querySemir.Select(o => o.Kvid);
                //var semirKvids = conn.Select<Guid>(querySemir);

                var semirs = conn.Select<ThirdReportOrder>(o => o.ThirdType == typeof(SemirOrder).FullName & o.IsUploaded == false & o.IsAccepted == true);
                if (semirs.IsNullOrEmpty())
                {
                    TaskContext.Message = "暂无数据处理";
                    return true;
                }
                //var semirKvids = semirs.ConvertAll(o => o.Kvid);

                var instance = Activator.CreateInstance(typeof(SemirOrder)) as IEntityIsThirdReportOrder;
                var successed = 0;
                var skip = 0;

                //var result = instance.OnUploadThirdContract(semir.Kvid);
                //if (result) successed++;
                //else skip++;
                //TaskContext.Message = $"待回传订单：{semir.SerialNumber}，包含报告:{semir.ReportIds} , 成功回传：{successed}条，跳过{skip}条。";
                //return true;
                foreach (var order in semirs)
                {
                    var thirdDetails = conn.Select<ThirdReportOrderDetail>(o => o.ThirdOrderKvid == order.Kvid);
                    if (thirdDetails.IsNullOrEmpty())
                    {
                        skip++;
                        continue;
                    }
                    var reports = conn.SelectByIds<Report>(thirdDetails.ConvertAll(o => o.ReportKvid));
                    if (reports.IsNullOrEmpty())
                    {
                        skip++;
                        continue;
                    }
                    if (reports.Exists(o => o.Status < (int)ReportStatusType.ReportIssue))
                    {
                        skip++;
                        continue;
                    }
                    if (reports.Exists(o => o.TestJudgement == "不合格"))
                    {
                        skip++;
                        continue;
                    }

                    var result = instance.OnUploadThirdReport(order.Kvid);
                    if (result) successed++;
                    else skip++;
                }
                TaskContext.Message = $"合计待回传订单：{semirs.Count}条，成功回传：{successed}条，跳过{skip}条。";
                return true;
            }
            catch (Exception ex)
            {
                TaskContext.Message = $"错误：{ex.Message}";
                return false;
            }
            finally
            {
                var thread = KiviiContext.NewThread(() =>
                {
                    //没有关联报告的 自动执行符合条件的关联
                    SemirExtension.LinkReports();
                });
                thread.Start();
            }
        }

        public void Dispose()
        {
        }
    }
}
